﻿Public Class Form1
    Private grid(10, 10) As Integer
    Private fishList As New List(Of Fish)
    Private sharkList As New List(Of Shark)
    Private rnd As New Random()

    Private Const WATER_CELL As Integer = 0
    Private Const FISH_CELL As Integer = 1
    Private Const SHARK_CELL As Integer = 2

    Protected Overrides Sub OnPaint(e As PaintEventArgs)
        MyBase.OnPaint(e)
        DrawGrid(e.Graphics)
    End Sub

    Private Sub DrawGrid(g As Graphics)
        Dim cellWidth As Integer = Me.ClientSize.Width / grid.GetLength(0)
        Dim cellHeight As Integer = Me.ClientSize.Height / grid.GetLength(1)

        For x As Integer = 0 To grid.GetLength(0) - 1
            For y As Integer = 0 To grid.GetLength(1) - 1
                Select Case grid(x, y)
                    Case WATER_CELL
                        g.FillRectangle(Brushes.Blue, x * cellWidth, y * cellHeight, cellWidth, cellHeight)
                    Case FISH_CELL
                        g.FillRectangle(Brushes.Green, x * cellWidth, y * cellHeight, cellWidth, cellHeight)
                    Case SHARK_CELL
                        g.FillRectangle(Brushes.Gray, x * cellWidth, y * cellHeight, cellWidth, cellHeight)
                End Select
            Next
        Next
    End Sub

    Private Sub InitializeSimulation()
        ' Randomly populate the grid with fish and sharks
        For i As Integer = 0 To 20
            AddFish(rnd.Next(0, grid.GetLength(0)), rnd.Next(0, grid.GetLength(1)))
        Next

        For i As Integer = 0 To 10
            AddShark(rnd.Next(0, grid.GetLength(0)), rnd.Next(0, grid.GetLength(1)))
        Next

        Timer1.Interval = 1000 ' Update every second
        Timer1.Start()
    End Sub

    Private Sub AddFish(x As Integer, y As Integer)
        If grid(x, y) = WATER_CELL Then
            grid(x, y) = FISH_CELL
            fishList.Add(New Fish(x, y))
        End If
    End Sub

    Private Sub AddShark(x As Integer, y As Integer)
        If grid(x, y) = WATER_CELL Then
            grid(x, y) = SHARK_CELL
            sharkList.Add(New Shark(x, y))
        End If
    End Sub

    Private Sub Timer1_Tick(sender As Object, e As EventArgs) Handles Timer1.Tick
        UpdateSimulation()
        Me.Invalidate() ' Redraw the form
    End Sub

    Private Sub UpdateSimulation()
        For i As Integer = sharkList.Count - 1 To 0 Step -1
            Dim shark = sharkList(i)
            MoveShark(shark)
            If Not shark.IsAlive Then
                sharkList.RemoveAt(i)
            End If
        Next

        For i As Integer = fishList.Count - 1 To 0 Step -1
            Dim fish = fishList(i)
            MoveFish(fish)
        Next
    End Sub

    Private Sub MoveFish(fish As Fish)
        ' Implement fish movement logic
        Dim newX As Integer = (fish.X + rnd.Next(-1, 2)) ' Calculate new X position
        Dim newY As Integer = (fish.Y + rnd.Next(-1, 2)) ' Calculate new Y position

        ' Ensure newX and newY are within bounds
        If newX < 0 Then newX = grid.GetLength(0) - 1
        If newX >= grid.GetLength(0) Then newX = 0
        If newY < 0 Then newY = grid.GetLength(1) - 1
        If newY >= grid.GetLength(1) Then newY = 0

        If grid(newX, newY) = WATER_CELL Then
            grid(fish.X, fish.Y) = WATER_CELL
            fish.X = newX
            fish.Y = newY
            grid(newX, newY) = FISH_CELL
        End If
    End Sub

    Private Sub MoveShark(shark As Shark)
        ' Try finding a fish nearby to eat
        Dim foundFish As Boolean = False
        Dim potentialMoves As New List(Of (Integer, Integer)) ' list to store possible moves 

        ' Check adjacent cells (including diagonals) for fish
        For dx As Integer = -1 To 1
            For dy As Integer = -1 To 1
                If dx = 0 And dy = 0 Then Continue For ' Skip the current position
                Dim newX As Integer = (shark.X + dx + grid.GetLength(0)) Mod grid.GetLength(0)
                Dim newY As Integer = (shark.Y + dy + grid.GetLength(1)) Mod grid.GetLength(1)

                If grid(newX, newY) = FISH_CELL Then
                    ' If there's a fish, move to that position
                    grid(shark.X, shark.Y) = WATER_CELL  ' Leave current cell empty
                    shark.X = newX
                    shark.Y = newY
                    grid(newX, newY) = SHARK_CELL ' Place the shark in the new position
                    shark.Age = 0 ' Reset age if the shark eats
                    fishList.RemoveAll(Function(f) f.X = newX AndAlso f.Y = newY) ' Remove the eaten fish
                    foundFish = True
                    Exit For
                ElseIf grid(newX, newY) = WATER_CELL Then
                    ' Collect potential water moves
                    potentialMoves.Add((newX, newY))
                End If
            Next
            If foundFish Then Exit For
        Next

        If Not foundFish And potentialMoves.Count > 0 Then
            ' If no fish found, move to one of the potential water cells randomly
            Dim moveIndex As Integer = rnd.Next(potentialMoves.Count)
            Dim chosenMove = potentialMoves(moveIndex)

            grid(shark.X, shark.Y) = WATER_CELL  ' Leave current cell empty
            shark.X = chosenMove.Item1
            shark.Y = chosenMove.Item2
            grid(chosenMove.Item1, chosenMove.Item2) = SHARK_CELL ' Move to the new position
            shark.Age += 1 ' Increase age, as the shark did not eat
        ElseIf Not foundFish Then
            shark.Age += 1 ' Increase age if shark failed to move or find food
        End If

        ' Check for starvation and remove the shark if necessary
        If shark.Age > shark.StarvationTime Then
            shark.IsAlive = False ' Shark dies from starvation
        End If
    End Sub

    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        InitializeSimulation()
    End Sub
End Class
